% MATLAB script main\_LFE\_no\_reac.m 

% Solving a simple 2nd-order elliptic Dirichlet problem by means of 
% linear finite elements on a triangular mesh.
F_HANDLE = @(x,varargin) 1; % Right-hand side function $f(\Bx)$
GD_HANDLE = @(x,varargin) 0; % Dirichlet data $g(\Bx)$ \prblabel{code:main_no_reac:line:gd}
 
% Load vertex coordinates and vertex indices for triangles from 
% file, see \lectref[Ex.~]{ex:trimeshfile}
Mesh = load_Mesh('Coord_Polygon_6.dat', 'Elem_Polygon_6.dat');

% Element flags. This is not used, but must be present.
Mesh.ElemFlag = ones(size(Mesh.Elements,1),1);

% Initialization of the Edges and Vert2Edge records of Mesh, see 
% \lehrfemsect{sect:MDS} and \lectsect{sec:meshdata}, in particular
% \lectref[Ex.]{ex:matlabmeshdata}.
Mesh = add_Edges(Mesh);

% Extract indices of edges located on the boundary, that is, edges 
% that abut only a single triangle. The edge indices refer to the 
% position of the edge in the Mesh.Edges record of the Mesh data 
% structure, see \lehrfemsect{sect:MDS} for details.
Loc = get_BdEdges(Mesh);

% Initialize array flagging boundary edges.
Mesh.BdFlags = zeros(size(Mesh.Edges,1), 1); 
Mesh.BdFlags(Loc) = -1; % \prblabel{code:main_no_reac:line:1}

% Assemble Galerkin matrix and load vector, see \lehrfemch{chap:assem}
% and \lectsect{sec:assembly}.  Initialization of \emph{sparse} Galerkin matrix;
% the second argument is a handle to a MATLAB function providing the 
% element matrix, see \lectdef{def:locstiff}. Note that \texttt{A} will be the 
% Galerkin matrix for the full space $\CS_1^0(\CM)$ also taking into account 
% basis functions on the boundary!
A = assemMat_LFE(Mesh, @STIMA_Lapl_LFE); % \prblabel{code:main_no_reac:line:am}

% Build right-hand side vector $\vec{\Bvarphi}$ using a 7-point quadrature rule
% of order 6, see \lectrem{rem:lfemquad} and \lehrfemch{chap:quad_rule}. F\_HANDLE 
% passes a handle to the source function $f(\Bx)$. Basis functions 
% associated with vertices on $\partial\Omega$ are taken into account.
L = assemLoad_LFE(Mesh,P7O6(), F_HANDLE);

% Set components of $\vec{\Bmu}$ ($\leftrightarrow$ \texttt{U}) for basis functions located on $\partial\Omega$ to 
% the corresponding value of the Dirichlet data $g$ (passed through 
% G\_HANDLE. Also returns a vector FreeDofs of indices of vertices 
% located in the interior of $\Omega$. This functions expects a valid 
% BdFlags field in Mesh. The argument '-1' matches the flag for 
% boundary edges set in Line \prbref{code:main_no_reac:line:1}.
[U,FreeDofs] = assemDir_LFE(Mesh, -1, GD_HANDLE);

% Modify right-hand side $\vec{\Bvarphi}$ in order to take into account Dirichlet 
% data. This is explained in \lectsect{sec:essbdc}. Note that the length
% of both L and U agrees with the total number of vertices in the 
% mesh.
L = L - A*U;

% Solve the linear system using MATLAB's $\backslash$--operator. Only the
% coefficients for basis functions belonging to interior vertices 
% are involved.
U(FreeDofs) = A(FreeDofs,FreeDofs)\L(FreeDofs);

% Plot solution, see \lehrfemsect{sect:plot}.
plot_LFE(U,Mesh); colorbar; % \prblabel{code:main_no_reac:line:plot}

% Clear memory
clear all;
